# -*- coding: utf-8 -*-

# Automatically generated - don't edit.
# Use `python setup.py build_ui` to update it.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MultiLocaleSelector(object):
    def setupUi(self, MultiLocaleSelector):
        MultiLocaleSelector.setObjectName("MultiLocaleSelector")
        MultiLocaleSelector.setWindowModality(QtCore.Qt.WindowModal)
        MultiLocaleSelector.resize(507, 284)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(MultiLocaleSelector.sizePolicy().hasHeightForWidth())
        MultiLocaleSelector.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(MultiLocaleSelector)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label = QtWidgets.QLabel(MultiLocaleSelector)
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        self.selected_locales = QtWidgets.QListWidget(MultiLocaleSelector)
        self.selected_locales.setObjectName("selected_locales")
        self.verticalLayout_2.addWidget(self.selected_locales)
        self.horizontalLayout.addLayout(self.verticalLayout_2)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem)
        self.move_up = QtWidgets.QToolButton(MultiLocaleSelector)
        self.move_up.setText("")
        icon = QtGui.QIcon.fromTheme(":/images/16x16/go-up.png")
        self.move_up.setIcon(icon)
        self.move_up.setObjectName("move_up")
        self.verticalLayout_3.addWidget(self.move_up)
        self.add_locale = QtWidgets.QToolButton(MultiLocaleSelector)
        self.add_locale.setText("")
        icon = QtGui.QIcon.fromTheme(":/images/16x16/go-previous.png")
        self.add_locale.setIcon(icon)
        self.add_locale.setObjectName("add_locale")
        self.verticalLayout_3.addWidget(self.add_locale)
        self.remove_locale = QtWidgets.QToolButton(MultiLocaleSelector)
        self.remove_locale.setText("")
        icon = QtGui.QIcon.fromTheme(":/images/16x16/go-next.png")
        self.remove_locale.setIcon(icon)
        self.remove_locale.setObjectName("remove_locale")
        self.verticalLayout_3.addWidget(self.remove_locale)
        self.move_down = QtWidgets.QToolButton(MultiLocaleSelector)
        self.move_down.setText("")
        icon = QtGui.QIcon.fromTheme(":/images/16x16/go-down.png")
        self.move_down.setIcon(icon)
        self.move_down.setObjectName("move_down")
        self.verticalLayout_3.addWidget(self.move_down)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem1)
        self.horizontalLayout.addLayout(self.verticalLayout_3)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label_2 = QtWidgets.QLabel(MultiLocaleSelector)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_4.addWidget(self.label_2)
        self.available_locales = QtWidgets.QListWidget(MultiLocaleSelector)
        self.available_locales.setObjectName("available_locales")
        self.verticalLayout_4.addWidget(self.available_locales)
        self.horizontalLayout.addLayout(self.verticalLayout_4)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.button_box = QtWidgets.QDialogButtonBox(MultiLocaleSelector)
        self.button_box.setOrientation(QtCore.Qt.Horizontal)
        self.button_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Save)
        self.button_box.setObjectName("button_box")
        self.verticalLayout.addWidget(self.button_box)

        self.retranslateUi(MultiLocaleSelector)
        QtCore.QMetaObject.connectSlotsByName(MultiLocaleSelector)

    def retranslateUi(self, MultiLocaleSelector):
        _translate = QtCore.QCoreApplication.translate
        MultiLocaleSelector.setWindowTitle(_("Locale Selector"))
        self.label.setText(_("Selected Locales"))
        self.move_up.setToolTip(_("Move selected locale up"))
        self.add_locale.setToolTip(_("Add to selected locales"))
        self.remove_locale.setToolTip(_("Remove selected locale"))
        self.move_down.setToolTip(_("Move selected locale down"))
        self.label_2.setText(_("Available Locales"))
