# -*- coding: utf-8 -*-
#
# Picard, the next-generation MusicBrainz tagger
#
# Copyright (C) 2007 Lukáš Lalinský
# Copyright (C) 2014, 2018, 2020 Laurent Monin
# Copyright (C) 2017 Sambhav Kothari
# Copyright (C) 2018 Shen-Ta Hsieh
# Copyright (C) 2018-2020 Philipp Wolfer
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.


# List of available user interface languages
UI_LANGUAGES = [
    # ('af', 'Afrikaans', N_('Afrikaans')),
    ('ar', 'العربية', N_('Arabic')),
    # ('ast', 'Asturian', N_('Asturian')),
    # ('bg', 'Български', N_('Bulgarian')),
    ('ca', 'Català', N_('Catalan')),
    ('cs', 'Čeština', N_('Czech')),
    # ('cy', 'Cymraeg', N_('Welsh')),
    ('da', 'Dansk', N_('Danish')),
    ('de', 'Deutsch', N_('German')),
    # ('de_CH', 'Deutsch (Schweiz)', N_('German (Switzerland)')),
    ('el', 'ελληνικά', N_('Greek')),
    ('en', 'English', N_('English')),
    ('en_AU', 'English (Australia)', N_('English (Australia)')),
    ('en_CA', 'English (Canada)', N_('English (Canada)')),
    ('en_GB', 'English (United Kingdom)', N_('English (United Kingdom)')),
    # ('eo', 'Esperanto', N_('Esperanto')),
    ('es', 'Español', N_('Spanish')),
    ('et', 'Eesti', N_('Estonian')),
    # ('fa', 'فارسی', N_('Persian')),
    ('fi', 'Suomi', N_('Finnish')),
    # ('fo', 'Føroyskt', N_('Faroese')),
    ('fr', 'Français', N_('French')),
    ('fr_CA', 'Français (Canada)', N_('French (Canada)')),
    # ('fy', 'Frysk', N_('Frisian')),
    ('gl', 'Galego', N_('Galician')),
    ('he', 'עברית', N_('Hebrew')),
    # ('hi', 'हिन्दी', N_('Hindi')),
    ('hu', 'Magyar', N_('Hungarian')),
    # ('id', 'Bahasa Indonesia', N_('Indonesian')),
    ('is', 'Íslenska', N_('Icelandic')),
    ('it', 'Italiano', N_('Italian')),
    ('ja', '日本語', N_('Japanese')),
    # ('kn', 'ಕನ್ನಡ', N_('Kannada')),
    ('ko', '한국어', N_('Korean')),
    ('lt', 'Lietuvių', N_('Lithuanian')),
    ('ms_MY', 'Bahasa Melayu (Malaysia)', N_('Malay (Malaysia)')),
    ('nb', 'Norsk bokmål', N_('Norwegian Bokmål')),
    # ('nds', 'Plattdüütsch', N_('Low German')),
    ('nl', 'Nederlands', N_('Dutch')),
    ('oc', 'Occitan', N_('Occitan')),
    ('pl', 'Polski', N_('Polish')),
    ('pt', 'Português', N_('Portuguese')),
    ('pt_BR', 'Português (Brasil)', N_('Portuguese (Brazil)')),
    ('ro', 'Română', N_('Romanian')),
    ('ru', 'Pyccĸий', N_('Russian')),
    # ('sco', 'Scots leid', N_('Scots')),
    ('sk', 'Slovenčina', N_('Slovak')),
    ('sl', 'Slovenščina', N_('Slovenian')),
    ('sq', 'Shqip', N_('Albanian')),
    # ('sr', 'Србин', N_('Serbian')),
    ('sv', 'Svenska', N_('Swedish')),
    # ('ta', 'தமிழ்', N_('Tamil')),
    ('tr', 'Türkçe', N_('Turkish')),
    ('uk', 'Украї́нська', N_('Ukrainian')),
    ('zh_CN', '中文（中国大陆）', N_('Chinese (China)')),
    ('zh_TW', '中文（台灣）', N_('Chinese (Taiwan)')),
]
